#ifndef __TRANSXFUNCTION_H
#define __TRANSXFUNCTION_H

#include "mfdfunction.h"
#include "transxstate.h"
#include "parser.h"
#include "mfdvarhandler.h"
#include "intercept.h"
#include "graph.h"


class TransXFunction: public MFDFunction
{
private:
	int viewmode;
	int numberviews;
	PARSER parser;
	char helpstring1[40],helpstring2[40],helpstring3[40],helpstring4[40],helpstring5[40];
protected:
	OBJHANDLE hmajor, hminor, hmajtarget, hcraft, hbase;//Standard set of OBJHANDLES for the TransX MFD
	double gravbodyratio; //Specific computation associated with hmajor+hminor
	double simstartMJD; //Time at which current scenario commenced
	class MFDvarhandler vars; //Variable set associated with this MFDFunction
	class transxstate *state; //Pointer to calling transxstate
	int W,H, linespacing; //Width and height of calling MFD


	//General purpose routines used in multiple transxfunctions
	bool encounter (HDC hDC, GRAPH &locgraph,OBJHANDLE hobject, OBJHANDLE satellitebodyhandle, INTERCEPT *intercept, DWORD pencolour);
	void setnumberviews(int tnumberviews);
	HPEN SelectDefaultPen(HDC hDC, int value);
	void setviewmode(int tviewmode);


public:
	MFDvarhandler* getvariablehandler();//Passes pointer to variable handler
	TransXFunction(class transxstate *tstate, OBJHANDLE thmajor, OBJHANDLE thminor, OBJHANDLE thtarget, OBJHANDLE thcraft, OBJHANDLE thbase);//Constructor
	virtual ~TransXFunction();//Destructor
	virtual void restoreself(FILEHANDLE scn);//Attempts to restore previous saved state
	virtual void saveself(FILEHANDLE scn);//Saves current state to file
	void findfinish(FILEHANDLE scn);//Find end of function
	bool loadhandle(FILEHANDLE scn,OBJHANDLE *handle);//Loads an objecthandle from a file
	void savehandle(FILEHANDLE scn, OBJHANDLE handle);//Saves handle to scenario file
	void savevector(FILEHANDLE scn, VECTOR3 &vector);
	void savedouble(FILEHANDLE scn, double savenumber);
	bool loadint(FILEHANDLE scn, int *loadedint);
	bool loaddouble(FILEHANDLE scn, double *loadednumber);
	bool loadvector(FILEHANDLE scn, VECTOR3 *loadedvector);
	void saveorbit(FILEHANDLE scn, const ORBIT &saveorbit);//Saves an orbit structure
	bool loadorbit(FILEHANDLE scn, ORBIT *loadorbit);//Loads an orbit structure
	void inc_viewmode(); //Called when viewmode changed
	int getviewmode();
	MFDvariable *getcurrentvariable();
	void gethandles(OBJHANDLE *thmajor, OBJHANDLE *thminor, OBJHANDLE *thtarget, OBJHANDLE *thcraft, OBJHANDLE *thbase); //Gets handles
	void sethandles(OBJHANDLE thmajor, OBJHANDLE thminor, OBJHANDLE thtarget, OBJHANDLE thcraft, OBJHANDLE thbase);
	void sethmajor(OBJHANDLE handle);
	bool sethminor(OBJHANDLE handle);
	virtual bool initialisevars() = 0;
	bool sethmajtarget(OBJHANDLE handle);
	void sethcraft(OBJHANDLE handle);
	void sethbase(OBJHANDLE handle);
	void sethelp(char *help1,char *help2,char *help3,char *help4,char *help5);
	void gethelp(char *help1,char *help2,char *help3,char *help4,char *help5) const;
};

#endif