#define STRICT

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "c:\program files\orbiter\orbitersdk\include\orbitersdk.h"
#include "mfd.h"
#include "parser.h"
#include "mfdvarhandler.h"

void MFDvarhandler::deleteall()
{
	for (int a=0;a<lastvariable;a++)
	{
		delete pointers[a];
		pointers[a]=NULL;
	}
	lastvariable=0;
	for (a=0;a<MAXVIEWMODES;a++)
	{
		startvarpointer[a]=endvarpointer[a]=0;
	}
	currvariable=-1;
}

void MFDvarhandler::saveallvariables(FILEHANDLE scn)
{
	char buffer[80];
	for (int a=0;a<lastvariable;a++)
	{
		pointers[a]->getname(buffer);
		oapiWriteLine(scn,buffer);
		pointers[a]->getsaveline(buffer);
		oapiWriteLine(scn,buffer);
	}
	strcpy(buffer,"Finvars");
	oapiWriteLine(scn,buffer);
}

bool MFDvarhandler::loadallvariables(FILEHANDLE scn)
{
	PARSER parser;
	char *buffer,*member;
	char namebuffer[20];
	int length,tadjmode;
	for (int a=0;a<lastvariable;a++)
	{
		if (!oapiReadScenario_nextline(scn,buffer)) return false;
		pointers[a]->getname(namebuffer);
		if (strcmp(buffer,namebuffer)==0)
		{
			//Name matches
			if (!oapiReadScenario_nextline(scn,buffer)) return false;//Unexpected end
			parser.parseline(buffer);
			if (!parser.getlineelement(0,&member,&length)) return false;//No required element
			sscanf(member,"%i",&tadjmode);
			pointers[a]->setadjmode(tadjmode);//Set the adjustment mode
			if (!parser.getlineelement(1,&member,&length)) return false;//No required element
			pointers[a]->loadvalue(member);//Set the variable to the saved value
		}
		else
		{
			//No match
			if (strcmp(member,"Finvars")==0) return true;//Somehow reached end of section - return anyway
			a--;//Try to load the same variable again.
			//Routine copes with additional variables, but fails on deletions or renames
		}
	}
	//Now finished variables, look for the end label
	do
	{
		if (!oapiReadScenario_nextline(scn,buffer)) return false;
	}
	while (strcmp(buffer,"Finvars")!=0);
	return true;
}


MFDvarhandler::MFDvarhandler()
{
	lastvariable=0;
	for (int a=0;a<MAXPOINTERS;a++)
	{
		pointers[a]=NULL;
	}
	for (a=0;a<MAXVIEWMODES;a++)
	{
		startvarpointer[a]=endvarpointer[a]=0;
	}
	currvariable=-1;	
}


MFDvarhandler::~MFDvarhandler()
{
	for (int a=0;a<lastvariable;a++)
	{
		delete pointers[a];
	}
}


void MFDvarhandler::setviewrange(int viewmode, int start, int end)
{
	if (viewmode<MAXVIEWMODES)
	{
		if (start>-1 && start<lastvariable)
		{
			startvarpointer[viewmode]=start;
		}
		else
		{
			startvarpointer[viewmode]=0;
		}
		if (end>-1 && end<=lastvariable)
		{
			endvarpointer[viewmode]=end;
		}
		else
		{
			endvarpointer[viewmode]=0;
		}
	}
}

void MFDvarhandler::setnextcurrent(int viewmode)
{
	++currvariable;
	if (currvariable>endvarpointer[viewmode]-1 || currvariable<startvarpointer[viewmode])
	{
		currvariable=startvarpointer[viewmode];
	}
	return;
}


void MFDvarhandler::setprevcurrent(int viewmode)
{
	--currvariable;
	if (currvariable<startvarpointer[viewmode] || currvariable>endvarpointer[viewmode]-1)
	{
		currvariable=endvarpointer[viewmode]-1;
	}
}


MFDvariable* MFDvarhandler::getcurrent(int viewmode)
{
	if (currvariable<startvarpointer[viewmode] || currvariable>=endvarpointer[viewmode])
	{
		return NULL;
	}
	return pointers[currvariable];
}


bool MFDvarhandler::newvarangle(char *vname, int vadjmode, double vvalue, double vcosvalue, double vsinvalue, double vmin, double vmax, double vincrement, bool vloop)
{
	MFDvarangle* varangle;
	varangle=new MFDvarangle(vname, vadjmode, vvalue, vcosvalue, vsinvalue, vmin, vmax, vincrement, vloop);
	pointers[lastvariable++]=varangle;
	return (varangle!=NULL);
}

bool MFDvarhandler::newvarMJD(char *vname, int vadjmode, double vvalue, double vmin, double vmax, double vincrement, double vlogborder)
{
	MFDvarMJD* varmjd;
	if (vvalue<1) vvalue=oapiGetSimMJD();
	varmjd=new MFDvarMJD(vname, vadjmode, vvalue, vmin, vmax, vincrement, vlogborder);
	pointers[lastvariable++]=varmjd;
	return (varmjd!=NULL);
}



bool MFDvarhandler::newvarfloat(char *vname, int vadjmode, double vvalue, double vmin, double vmax, double vincrement, double vlogborder)
{
	MFDvarfloat* varfloat;
	varfloat=new MFDvarfloat(vname, vadjmode, vvalue, vmin, vmax, vincrement, vlogborder);
	pointers[lastvariable++]=varfloat;
	return (varfloat!=NULL);
}

bool MFDvarhandler::newvardiscrete(char *vname, int vvalue, int vlimit, char *st1, char *st2, char *st3, char *st4, char *st5)
{
	MFDvardiscrete* vardiscrete;
	vardiscrete=new MFDvardiscrete(vname, vvalue, vlimit, st1, st2, st3, st4, st5);
	pointers[lastvariable++]=vardiscrete;
	return (vardiscrete!=NULL);
}

bool MFDvarhandler::newvarbody(char *vname, char *vlabel)
{
	MFDvarbody* varbody;
	varbody=new MFDvarbody(vname, vlabel);
	pointers[lastvariable++]=varbody;
	return (varbody!=NULL);
}

MFDvariable *MFDvarhandler::getpointer(int index) const
{
	if (index<0 || index>lastvariable)
		index=0; //Not perfect, but it'll stop some crashes
	return pointers[index];
}

void MFDvarhandler::getsincos(int index, double *sin, double *cos) const
{
	pointers[index]->getsincos(sin, cos);
}

void MFDvarhandler::setvalue(int index, double tvalue) const
{
	pointers[index]->setvalue(tvalue);
}

void MFDvarhandler::sethandle(int index, OBJHANDLE thandle) const
{
	pointers[index]->sethandle(thandle);
}

void MFDvarhandler::setivalue(int index, int tvalue) const
{
	pointers[index]->setivalue(tvalue);
}

void MFDvarhandler::gethelpstrings(int index, char *help1,char *help2)
{
	pointers[index]->gethelpstrings(help1,help2);
}

void MFDvarhandler::sethelpstrings(int index, char *help1, char *help2)
{
	pointers[index]->sethelpstrings(help1, help2);
}

double MFDvarhandler::getcos(int index) const
{
	return pointers[index]->getcos();
}

OBJHANDLE MFDvarhandler::gethandle(int index) const
{
	return pointers[index]->gethandle();
}

double MFDvarhandler::getsin(int index) const
{
	return pointers[index]->getsin();
}

double MFDvarhandler::getvalue(int index) const
{
	return pointers[index]->getvalue();
}

int MFDvarhandler::getivalue(int index) const
{
	return pointers[index]->getivalue();
}

int MFDvarhandler::getlastvariable() const
{
	return lastvariable;
}

