#define STRICT

#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "c:\program files\orbiter\orbitersdk\include\orbitersdk.h"
#include "mfd.h"
#include "mfdvartypes.h"





MFDvarbody::MFDvarbody(char *vname, char *vlabel)
{
	strcpy(name,vname);
	strcpy(label,vlabel);
	adjmode=0;
	pointer=NULL;
}

void MFDvarbody::getsaveline(char *buffer) const
{
	getsaveline1(buffer);
	char tbuffer[30];
	if (pointer!=NULL)
	{
		oapiGetObjectName(pointer,tbuffer,30);
	}
	else
	{
		strcpy(tbuffer,"NULL");
	}
	strcat(buffer,tbuffer);
}

bool MFDvarbody::loadvalue(char *buffer)
{
	if (strcmp(buffer,"NULL")==0)
	{
		pointer=NULL;
	}
	else
	{
		pointer=oapiGetObjectByName(buffer);
	}
	return true;
}

bool MFDvarbody::SetVariableBody(char *str)
{
	OBJHANDLE temp=oapiGetGbodyByName(str);
	if (temp !=NULL)
	{
		pointer=temp;
		return true;
	}
	else
	{
		return false;
	}

}


bool MFDvarbody::show(HDC hDC, int width, int line) const
{
	char buffer[20];
	int linecentre=(int) width/2;
	int linepos=3*line;
	strcpy(buffer,name);
	int length=strlen(buffer);
	TextOut(hDC, linecentre, linepos, buffer, length);
	showadjustment(hDC,width, line);
	linepos+=line+line;
	if (pointer!=NULL)
	{
		oapiGetObjectName(pointer,buffer,20);
	}
	else
	{
		return true;
	}
	length=strlen(buffer);
	TextOut(hDC,linecentre,linepos,buffer,length);
	return true;
}

bool MFDvarbody::flip_variable()
{
	oapiOpenInputBox(label,SelectVariableBody, 0, 20, (void*)this);
	return true;
}

void MFDvarbody::sethandle(OBJHANDLE tpointer)
{
	pointer=tpointer;
}

OBJHANDLE MFDvarbody::gethandle() const
{
	return pointer;
}

MFDvarbody::~MFDvarbody() {}

/* 

  MFDvarfloat class

*/

MFDvarfloat::MFDvarfloat(char *vname, int vadjmode, double vvalue, double vmin, double vmax, double vincrement, double vlogborder)
{
	strcpy(name,vname);
	adjmode=vadjmode;
	value=vvalue;
	min=vmin;
	max=vmax;
	increment=vincrement;
	logborder=vlogborder;
}

bool MFDvarfloat::show(HDC hDC, int width, int line) const
{
	char buffer[20];
	int linecentre=(int) width/2;
	int linepos= 3*line;
	strcpy(buffer,name);
	int length=strlen(buffer);
	TextOut(hDC, linecentre, linepos, buffer, length);
	showadjustment(hDC, width, line);
	linepos+=line+line;
	TextShow(hDC, " ", linecentre, linepos, value);
	return true;
}

void MFDvarfloat::getsaveline(char *buffer) const
{
	getsaveline1(buffer);
	char tbuffer[30];
	sprintf(tbuffer," %g",value);
	strcat(buffer,tbuffer);
}

bool MFDvarfloat::loadvalue(char *buffer)
{
	value=atof(buffer);
	return true;
}



bool MFDvarfloat::inc_variable()
{
	double temp=value;
	double adjuster=0;
	switch (adjmode){
	case 1:
		adjuster=0.1;
		break;
	case 2:
		adjuster=0.01;
		break;
	case 3:
		adjuster=0.001;
		break;
	case 4:
		adjuster=0.0001;
		break;
	case 5:
		adjuster=0.00001;
		break;
	}
	if (temp>logborder || temp<-logborder)
	{
		temp+=fabs(temp)*adjuster*increment;
	}
	else
	{
		temp+=logborder*adjuster*increment;
	}
	if (temp>max) temp=max;
	value=temp;
	return true;
}

bool MFDvarfloat::dec_variable()
{
	double temp=value;
	double adjuster=0;
	switch (adjmode){
	case 1:
		adjuster=0.1;
		break;
	case 2:
		adjuster=0.01;
		break;
	case 3:
		adjuster=0.001;
		break;
	case 4:
		adjuster=0.0001;
		break;
	case 5:
		adjuster=0.00001;
		break;
	}
	if (temp>logborder || temp<-logborder)
	{
		temp-=fabs(temp)*adjuster*increment;
	}
	else
	{
		temp-=logborder*adjuster*increment;
	}
	if (temp<min) temp=min;
	value=temp;
	return true;
}

void MFDvarfloat::setvalue(double tvalue)
{
	value=tvalue;
}

double MFDvarfloat::getvalue() const
{
	return value;
}

MFDvarfloat::~MFDvarfloat() {}

/*

  MFDvarMJD class

*/


MFDvarMJD::MFDvarMJD(char *vname, int vadjmode, double vvalue, double vmin, double vmax, double vincrement, double vlogborder)
:MFDvarfloat(vname, vadjmode, vvalue, vmin, vmax, vincrement, vlogborder)
{ }


bool MFDvarMJD::show(HDC hDC, int width, int line) const
{
	char buffer[20];
	int linecentre=(int) width/2;
	int linepos= 3*line;
	strcpy(buffer,name);
	int length=strlen(buffer);
	TextOut(hDC, linecentre, linepos, buffer, length);
	showadjustment(hDC, width, line);
	linepos+=line+line;
	length=sprintf(buffer,"%.4f", value);
	TextOut(hDC, linecentre, linepos, buffer, length);
	return true;
}


MFDvarMJD::~MFDvarMJD() {}

/*

  MFDvardiscrete class

*/



MFDvardiscrete::MFDvardiscrete(char *vname, int vvalue, int vlimit, char *st1, char *st2, char *st3, char *st4, char *st5)
{
	strcpy(name,vname);
	adjmode=0;
	value=vvalue;
	limit=vlimit;
	strcpy(label[0], st1);
	strcpy(label[1], st2);
	strcpy(label[2], st3);
	strcpy(label[3], st4);
	strcpy(label[4], st5);
}


bool MFDvardiscrete::show(HDC hDC, int width, int line) const
{
	char buffer[20];
	int linecentre=(int) width/2;
	int linepos= 3*line;
	strcpy(buffer,name);
	int length=strlen(buffer);
	TextOut(hDC, linecentre, linepos, buffer, length);
	showadjustment(hDC, width, line);
	length=strlen(label[value]);
	linepos+=line+line;
	TextOut(hDC, linecentre, linepos, label[value], length);
	return true;
}

void MFDvardiscrete::getsaveline(char *buffer) const
{
	getsaveline1(buffer);
	char tbuffer[30];
	sprintf(tbuffer," %i",value);
	strcat(buffer,tbuffer);
}

bool MFDvardiscrete::loadvalue(char *buffer)
{
	sscanf(buffer,"%i",&value);
	return true;
}


bool MFDvardiscrete::flip_variable()
{

	if (++value>limit) value=0;
	return true;
}


void MFDvardiscrete::setivalue(int tvalue)
{
	value=tvalue;
}

int MFDvardiscrete::getivalue() const
{
	return value;
}

MFDvardiscrete::~MFDvardiscrete() {}

/*

  MFDvarangle class

*/


MFDvarangle::MFDvarangle(char *vname, int vadjmode, double vvalue, double vcosvalue, double vsinvalue,
						 double vmin, double vmax, double vincrement, bool vloop)
{
	strcpy(name,vname);
	adjmode=vadjmode;
	value=vvalue;
	cosvalue=vcosvalue;
	sinvalue=vsinvalue;
	min=vmin;
	max=vmax;
	increment=vincrement;
	loop=vloop;
}

bool MFDvarangle::show(HDC hDC, int width, int line) const
{
	char buffer[20];
	int linecentre=(int) width/2;
	int linepos= 3*line;
	strcpy(buffer,name);
	int length=strlen(buffer);
	TextOut(hDC, linecentre, linepos, buffer, length);
	showadjustment(hDC, width, line);
	linepos+=line+line;
	length=sprintf(buffer,"%.4f'", value/PI*180);
	TextOut(hDC, linecentre, linepos, buffer, length);
	return true;
}

void MFDvarangle::getsaveline(char *buffer) const
{
	getsaveline1(buffer);
	char tbuffer[30];
	sprintf(tbuffer," %g",value);
	strcat(buffer,tbuffer);
}

bool MFDvarangle::loadvalue(char *buffer)
{
	value=atof(buffer);
	return true;
}


bool MFDvarangle::inc_variable()
{

	double temp=value;
	double adjuster=0;
	switch (adjmode){
	case 1:
		adjuster=0.1;
		break;
	case 2:
		adjuster=0.01;
		break;
	case 3:
		adjuster=0.001;
		break;
	case 4:
		adjuster=0.0001;
		break;
	case 5:
		adjuster=0.00001;
		break;

	}
	temp+=adjuster*increment;
	if (temp>max)
	{
		if (loop)
		{
			temp=min;
		}
		else
		{
			temp=max;
		}
	}
	value=temp;
	cosvalue=cos(temp);
	sinvalue=sin(temp);
	return true;
}

bool MFDvarangle::dec_variable()
{
	double temp=value;
	double adjuster=0;
	switch (adjmode){
	case 1:
		adjuster=0.1;
		break;
	case 2:
		adjuster=0.01;
		break;
	case 3:
		adjuster=0.001;
		break;
	case 4:
		adjuster=0.0001;
		break;
	case 5:
		adjuster=0.00001;
		break;
	}
	temp-=adjuster*increment;
	if (temp<min)
	{
		if (loop)
		{
			temp=max;
		}
		else
		{
			temp=min;
		}
	}
	value=temp;
	cosvalue=cos(temp);
	sinvalue=sin(temp);
	return true;
}
	

void MFDvarangle::getsincos(double *sin, double *cos) const
{
	*sin=sinvalue;
	*cos=cosvalue;
}

double MFDvarangle::getsin() const
{
	return sinvalue;
}

double MFDvarangle::getcos() const
{
	return cosvalue;
}

void MFDvarangle::setvalue(double tvalue)
{
	value=tvalue;
	cosvalue=cos(value);
	sinvalue=sin(value);
}

double MFDvarangle::getvalue() const
{
	return value;
}

MFDvarangle::~MFDvarangle() {}
