#ifndef __MFDVARTYPES_H
#define __MFDVARTPES_H

#include "mfdvariable.h"

class MFDvarbody : public MFDvariable {
protected:
	OBJHANDLE pointer;
	char label[40];
public:
	MFDvarbody(char *vname, char *vlabel);
	~MFDvarbody();
	void sethandle(OBJHANDLE tpointer);
	OBJHANDLE gethandle() const;
	bool flip_variable();
	bool SetVariableBody(char *str);
	virtual bool show(HDC hDC, int width, int line) const;
	virtual void getsaveline(char *buffer) const;
	virtual bool loadvalue(char *buffer);
};

class MFDvarfloat : public MFDvariable {
protected:
	double value; // Value of the variable
	double min; // Minimum legal value
	double max; //Maximum legal value
	double increment; //Increment fraction
	double logborder; // Number below which increment is linear scaled
public:
	bool inc_variable(); // Increase the variable
	bool dec_variable(); //Decrease the variable
	bool show(HDC hDC, int width, int line) const;
	double getvalue() const; //Get the value
	void setvalue(double tvalue);
	virtual void getsaveline(char *buffer) const;
	virtual bool loadvalue(char *buffer);
	MFDvarfloat(char *vname, int vadjmode, double vvalue, double vmin, double vmax, double vincrement, double vlogborder);
	~MFDvarfloat();
};

class MFDvarMJD: public MFDvarfloat {
public:
	bool show(HDC hDC, int width, int line) const;
	MFDvarMJD(char *vname, int vadjmode, double vvalue, double vmin, double vmax, double vincrement, double vlogborder);
	~MFDvarMJD();
};


class MFDvardiscrete: public MFDvariable {
private:
	char label[5][15]; //Label array
	int value; // Variable indicating the projection type
	int limit; // Highest permissible value - Note - 5 labels MAX!!
public:
	bool flip_variable(); //Change to next projection
	int getivalue() const; // Function to obtain value
	void setivalue(int tvalue);
	bool show(HDC hDC, int width, int line) const;
	virtual void getsaveline(char *buffer) const;
	virtual bool loadvalue(char *buffer);
	MFDvardiscrete(char *vname, int vvalue, int vlimit, char *st1, char *st2, char *st3, char *st4, char *st5);
	~MFDvardiscrete();
};

class MFDvarangle: public MFDvariable {
private:
	double value; // Value of the angle
	double cosvalue;//Cosine of the angle - calculated here to avoid repeated recalculation later
	double sinvalue; //Sin of the angle - as above
	double min; // Minimum legal value
	double max; // Maximum legal value
	double increment; // Increment value
	bool loop; //If true, then loops from min to max
public:
	bool inc_variable(); //Increase the variable
	bool dec_variable(); //Decrease the variable
	bool show(HDC hDC, int width, int line) const;
	double getvalue() const;
	void setvalue(double tvalue);
	void getsincos(double *sin, double *cos) const;
	double getsin() const;
	double getcos() const;
	virtual void getsaveline(char *buffer) const;
	virtual bool loadvalue(char *buffer);
	MFDvarangle(char *vname, int vadjmode, double vvalue, double vcosvalue, double vsinvalue, double vmin,
		double vmax, double vincrement, bool vloop);
	~MFDvarangle();
};

#endif