#include <windows.h>
#include <stdio.h>
#include <math.h>
#include "c:\program files\orbiter\orbitersdk\include\orbitersdk.h"
#include "mfd.h"
#include "ejectfunction.h"
#include "orbitejectfunction.h"
#include "transx.h"




OrbitEjectFunction::OrbitEjectFunction(class transxstate *tstate, OBJHANDLE thmajor, OBJHANDLE thminor, OBJHANDLE thtarget, OBJHANDLE thcraft, OBJHANDLE thbase)
: EjectFunction(tstate, thmajor, thminor, thtarget, thcraft, thbase)
{
	if (!(valid=initialisevars()))
	{
		state->setmfdstate(0);
	}
}

OrbitEjectFunction::~OrbitEjectFunction()
{
	vars.deleteall();
}


void OrbitEjectFunction::saveself(FILEHANDLE scn)
{
	vars.saveallvariables(scn);
	saveorbit(scn,basisorbit);
	savehandle(scn,hmajor);
	savehandle(scn,hminor);
	savehandle(scn,hmajtarget);
	oapiWriteScenario_int(scn,"Int",getviewmode());
	oapiWriteScenario_string(scn,"Finish","OrbitEjectFunction");
}

void OrbitEjectFunction::restoreself(FILEHANDLE scn)
{
	OBJHANDLE temp;
	if (!vars.loadallvariables(scn)) return;
	if (!loadorbit(scn,&basisorbit)) return;
	if (!loadhandle(scn,&temp)) return;
	if (temp!=NULL) sethmajor(temp);//hmajor must never be set null
	if (!loadhandle(scn,&temp)) return;
	sethminor(temp);
	if (!loadhandle(scn,&hmajtarget)) return;
	int temp2;
	if (!loadint(scn, &temp2)) return;
	setviewmode(temp2);
	findfinish(scn);
}



bool OrbitEjectFunction::doupdate(HDC hDC, int tw, int th)
// This is the orbit eject mode. It has two views, major and minor centred
// It covers the case that the craft is inside the RMIN sphere of influence
{
// Get the positions of the major relevant bodies in the global frame
	if (!valid) return true;
	W=tw;
	H=th;
	linespacing=H/20;
	hypormin.setinvalid();
	hypormaj.setinvalid();
	TransxMFD *mfdpointer=state->GetMFDpointer();
	graph=mfdpointer->graph;
	graphleft=mfdpointer->graphleft;
	graphright=mfdpointer->graphright;
	enum mode3variables {
		minprojection, minvieworb,base_orbit,provel, incvel, ejtime, majprojection, hypoorcraft, majvieworb};

	VECTOR3 gcraftpos, gmajorpos,temp;
	VECTOR3 gminorpos={1,1,1};//Just a value to initialise it to something

	// Get positions of bodies

	oapiGetGlobalPos(hmajor, &gmajorpos);
	if (hminor!=NULL)
	{
		oapiGetGlobalPos(hminor, &gminorpos);
		gmminor=oapiGetMass(hminor)*GRAVITY;
	}
	oapiGetGlobalPos(hcraft, &gcraftpos);
	temp=gmajorpos-gminorpos;
	soisize2=dotproduct(temp,temp)*gravbodyratio;


	bool invalidated;
	bool rettype=minormodevalidate(hDC,gminorpos,gcraftpos, "Fnctn:Orbit Eject",2,&invalidated);
	if (invalidated) return rettype;
	state->mfdstatevalidated=true;//All criteria needed to display this mode fulfilled

	//Craft now known to be inside SOI

	//Check if variables are initialised - if not, initialise them

	//Initialise orbit of minor around major
	rmin.init(hmajor, hminor);
	craftinrmin.init(hminor, hcraft);
	if (vars.getivalue(base_orbit)==1 || !basisorbit.isvalid())
	{
		basisorbit=craftinrmin;
		vars.setivalue(base_orbit,0);
	}
	bool goodcrab;
	VECTOR3 skewvector;

	//This section calculates the hypothetical ejection path.
	Getmode3hypo(&skewvector, &goodcrab, &ejectmjd);

	int wpos,hpos;
	bool showintercept;
	bool h_or_c=false;
	majtarget.init(hmajor, hmajtarget);


	if (getviewmode()==1)
	{
		// This is the RMin centred view
		graph.setprojection(rmin.getplanevector());//Sets initial projection
		showminorsoiview1(hDC, &graph, craftinrmin, hypormin, minprojection, minvieworb,false);

		// Label the mode
		wpos=W/2;
		hpos=0;
		TextOut(hDC,wpos, hpos, "Fnctn:Orbit Eject",17);
		return true;
	}
	//viewmode is 2 or 3
	
	// Calculate and show RMaj centred view
	if (craftinrmin.geteccentricity()>1) // There's a craft orbit in RMaj
	{
		// This gives us the craft's orbit in rmaj
		craftinrmin.minortomajor(rmin, &craftinrmaj, sqrt(soisize2));
	}
	// Get the orbit of the target, if target set
	//orbit majtarget(state->hmajor, state->hmajtarget);

	// Get the correct intercept 
	showintercept;
	h_or_c=false;
	if (vars.getivalue(hypoorcraft)==0) h_or_c=true;
	showintercept=updateinterception(hypormaj,craftinrmaj,majtarget,h_or_c);
	//Show the information gathered
	if (getviewmode()==2)
	{
		
		showminorsoiview2(hDC, &graph, h_or_c, showintercept, majprojection, majvieworb, true);
	}
	else
	{
		graphleft.setprojection(rmin.getplanevector());//Sets initial projection
		showminorsoiview1(hDC, &graphleft, craftinrmin, hypormin, minprojection, minvieworb,false);
		if (hypormin.isvalid())
		{
			double timeoffset=(ejectmjd-simstartMJD)*86400-craftinrmin.gettimestamp();
			VECTOR3 temppos,tempvel;
			craftinrmin.getcurrentvectors(&temppos,&tempvel);
			skewvector=skewvector+unitise(tempvel)*vars.getvalue(provel);
			VESSEL *pV=oapiGetFocusInterface();
			graphright.vectorpointdisplay(hDC, skewvector, mfdpointer, pV, false);
			wpos=W*11/20;
			hpos=H*8/20;
			TextOut(hDC,wpos,hpos,"Skew Guide Only",15);
			hpos+=linespacing;
			TextShow(hDC,"dT ",wpos,hpos,timeoffset);
		}
	}
	if (!goodcrab && hypormaj.isvalid())
	{
		wpos=0;
		hpos=H*15/20;
		TextOut(hDC,wpos,hpos,"Crab wasteful",13);
	}
	// Label the mode
	wpos=W/2;
	hpos=0;
	TextOut(hDC,wpos, hpos, "Fnctn:Orbit Eject",17);
	return true;
	
}


bool OrbitEjectFunction::initialisevars()

// Initialise MFDvariable objects used by mode 3
{
	enum mode3variables {
		minprojection, minvieworb,base_orbit,provel, incvel, ejtime, majprojection, hypoorcraft, majvieworb};
		
	
	vars.deleteall();
	bool test=true;
	test&=vars.newvardiscrete("Projection",0,3, "Ecliptic", "Craft", "Hypothetical","Minor Plane","");
	test&=vars.newvardiscrete("View Orbits",0,1,"All","Hypothetical","","","");
	test&=vars.newvardiscrete("Base Orbit",1,1,"++Updates","Updating","","","");
	test&=vars.newvarfloat("Prograde deltaV",1, 0,-1e8,1e8,0.05,1000);
	test&=vars.newvarfloat("Ch. plane deltaV",1,0,-1e8,1e8,0.05,1000);
	test&=vars.newvarMJD("Eject date",1,0,0,1e20,1e-8,1000000); 
	test&=vars.newvardiscrete("Projection",0,1, "Ecliptic","Craft","","","");
	test&=vars.newvardiscrete("Intercept with",0,1,"Hypothetical","Craft","","","");
	test&=vars.newvardiscrete("View Orbits",0,2,"All","Target","Craft","","");
	if (test==false)
	{
		vars.deleteall();
		valid=false;
		return false;
	}
	vars.setviewrange(1,0,6);
	vars.setviewrange(2,3,9);
	setnumberviews(2);
	setviewmode(2);
	valid=true;

	vars.sethelpstrings(minprojection,
		"Sets viewpoint from which you view",
		"current MFD graph.");
	vars.sethelpstrings(minvieworb,
		"Use to improve view of hypothetical",
		"orbit if small.");
	vars.sethelpstrings(base_orbit,
		"This orbit defines the baseline from",
		"which manoevres are calculated.");
	vars.sethelpstrings(provel,
		"Velocity increase from current",
		"(basis) orbit.");
	vars.sethelpstrings(incvel,
		"Use to 'crab' sideways, to change",
		"orbital plane. Hard to fly.");
	vars.sethelpstrings(ejtime,
		"Time of manoevre. Change to rotate",
		"manoevre around basis orbit.");
	vars.sethelpstrings(majprojection,
		"Sets viewpoint from which you view",
		"current MFD graph.");
	vars.sethelpstrings(hypoorcraft,
		"Choose what Intercept graphs and",
		"stats refer to.");
	vars.sethelpstrings(majvieworb,
		"Use to improve view of hypothetical",
		"or craft orbit if small.");
	sethelp(
		"ADVANCED USERS ONLY!",
		"Use to plan exit from planet/moon if",
		"already orbiting. More efficient and",
		"easier to use EJECT function instead",
		"and plan before taking off.");
	return true;
}


void OrbitEjectFunction::Getmode3hypo(VECTOR3 *skewvector, bool *goodcrab, double *ejectmjd)

// This obtains the current hypothetical orbit in rmin influence

{
	enum mode3variables {
		minprojection, minvieworb,base_orbit,provel, incvel, ejtime, majprojection, hypoorcraft, majvieworb};

	// New section
	double mjd=vars.getvalue(ejtime);//Gets the mjd for eject
	double currmjd=oapiGetSimMJD();// Gets the current mjd
	double soisize=sqrt(soisize2);
	*ejectmjd=mjd;
	double difftime=(mjd-simstartMJD)*86400-craftinrmin.gettimestamp();//Converts to seconds from orbit initialisation
	VECTOR3 ejradius, ejvel;
	craftinrmin.timetovectors(difftime, &ejradius, &ejvel);

	// Get eject vector
	VECTOR3 plane=craftinrmin.getplanevector();
	VECTOR3 forward=unitise(ejvel)*vars.getvalue(provel);
	VECTOR3 sideward=unitise(plane)*vars.getvalue(incvel);
	if (fabs(vars.getvalue(provel))+fabs(vars.getvalue(incvel))<1) return;
	ejvel=ejvel+forward+sideward;
	VECTOR3 temppos,tempvel;
	craftinrmin.getcurrentvectors(&temppos,&tempvel);
	*skewvector=sideward;
	//Generate new hypothetical orbits
	hypormin.init(ejradius, ejvel,(mjd-simstartMJD)*86400,gmminor);//Gets the new eject formula
	hypormin.minortomajor(rmin,&hypormaj,soisize); //Safe even if hypormin is still bound
	
	//Calculate whether the specified crabbing is efficient or not
	//Find the ratio of the velocity at eject point to velocity at soi boundary
	VECTOR3 soipos,soivel;
	double costhisoi=hypormin.radiustothi(soisize);
	if (costhisoi<1 && costhisoi>-1)
	{
		double sinthisoi=sqrt(1-costhisoi*costhisoi); //Taking positive root
		hypormin.thitovectors(costhisoi,sinthisoi,&soipos,&soivel);
		double vratio2=dotproduct(soivel,soivel)/dotproduct(ejvel,ejvel);
		double deltaratio2=dotproduct(sideward,sideward)/dotproduct(forward+sideward,forward+sideward);
		*goodcrab=(deltaratio2<vratio2);
	}
	else
	{
		*goodcrab=false;
	}
}




